import numpy as np
from astropy.convolution import Gaussian2DKernel, interpolate_replace_nans
from PIL import Image
import os
import sys
from scipy import fftpack

sys.path.insert(0, '../')

from utils import Color, mergeSingleColorChannelImagesAccordingToBayerFilter
import matplotlib.pyplot as plt

PREFIX = 'mean_rafael_230424_mean_'
X_STDDEV = 2

def getImageByColor(color):
    filePath = PREFIX + f'{color}.npy'
    image = np.load(filePath)
    return image

singleColorChannelImages = {color: getImageByColor(color) for color in Color}
multipleColorsImage = mergeSingleColorChannelImagesAccordingToBayerFilter(singleColorChannelImages)
image = multipleColorsImage

fft1 = fftpack.fftshift(fftpack.fft2(image))
originalFft1 = fft1.copy()
fft1 = abs(fft1)

# This example is intended to demonstrate how astropy.convolve and
# scipy.convolve handle missing data, so we start by setting the brightest
# pixels to NaN to simulate a "saturated" data set
height, width = fft1.shape
for x in range(width):
    fft1[height // 2, x] = np.nan

middleX = width // 2
RANGE = 1
for verticalLineX in [width // 4, width // 2, round(3 * width / 4)]:
    for y in range(height):
        for x in range(verticalLineX - RANGE, verticalLineX + RANGE + 1):
            fft1[y, x] = np.nan

# We smooth with a Gaussian kernel
kernel = Gaussian2DKernel(x_stddev = X_STDDEV)

# create a "fixed" image with NaNs replaced by interpolated values
fixedImage = interpolate_replace_nans(fft1, kernel)

figure, axes = plt.subplots(1, 2, sharex = True, sharey = True)

plt.suptitle('Attenuating FFT significant lines')
axes[0].set_title('Original FFT')
axes[0].imshow(np.log10(abs(originalFft1)))
axes[1].set_title('FFT with significant lines attenuated')
axes[1].imshow(np.log10(fixedImage))
plt.tight_layout()
plt.show()
#plt.imsave('fft.png', np.log10(fixedImage))

figure, axes = plt.subplots(1, 2, sharex = True, sharey = True)

def inverseFft(fft):
    ifft2 = np.real(fftpack.ifft2(fftpack.ifftshift(fft)))
    #ifft2 = np.maximum(0, np.minimum(ifft2, 255)) #* 255# / 255
    return ifft2

plt.suptitle('Rafael 23/04/24 PRNU mean denoiser with periodic patterns attenuated')
axes[0].set_title('Original PRNU')
axes[0].imshow(inverseFft(originalFft1))
axes[1].set_title('PRNU with periodic patterns attenuated')
axes[0].imshow(inverseFft(fixedImage))
plt.tight_layout()
plt.show()